package com.winhc.bigdata.spark.ng.chance

import com.alibaba.fastjson.JSON
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper
import com.winhc.bigdata.spark.utils.BaseUtil
import org.apache.commons.lang3.StringUtils
import org.apache.spark.internal.Logging

import scala.annotation.meta.{getter, setter}
import scala.collection.mutable
import scala.collection.mutable.ListBuffer

/**
 * @Author: π
 * @Date: 2021/9/28
 * @Description: 案源机会
 */
trait CompanyChanceHandle extends Logging {

  @getter
  @setter
  protected val is_inc: Boolean //false 为存量

  /**
   * 变更时间
   *
   * @param new_map
   * @return
   */
  protected def get_change_time(bizDate: String, new_map: Map[String, String]): String = {
    var res = bizDate
    if (bizDate.length == 10) {
      res = res.concat(" 00:00:00")
    }
    res
  }

  /**
   * 业务id
   *
   * @param rowkey
   * @return
   */
  protected def get_biz_id(rowkey: String, new_map: Map[String, String]): String = rowkey

  /**
   * 条件过滤
   *
   * @return
   */
  def get_conditional_filter(): String = ""

  /**
   * 加入标签
   *
   * @return
   */
  def get_label(new_map: Map[String, String]): String = ""

  /**
   * 原告-被告 集合
   *
   * @return
   */
  def get_party_set(new_map: Map[String, String]): Seq[String] = Seq("", "")

  protected def get_deleted(new_map: Map[String, String], key: String = "deleted"): String = {
    if (new_map == null) return "-1"
    val deleted = new_map.getOrElse(key, "-1")
    deleted
  }

  def filter: (String, String, Seq[String], Map[String, String], Map[String, String]) => Boolean = CompanyChanceUtils.default_filter

  def flat_map: (ChangeExtract) => Seq[CompanyChanceRecord]

  def getCompanyIds(new_data: Map[String, String]): Seq[String]

  protected def getEntity(json: String, id_key: String): Seq[String] = {
    if (StringUtils.isEmpty(json)) {
      Seq.empty
    } else {
      val array = JSON.parseArray(json)
      var list: mutable.Seq[String] = mutable.Seq.empty
      for (i <- 0 until array.size()) {
        val jSONObject = array.getJSONObject(i)
        val keyno = jSONObject.getString(id_key)
        list = list :+ keyno
      }
      list
    }
  }

  protected def getCompanyChanceRecord(change_extract: ChangeExtract
                                       , companyIds: Seq[String]
                                      ): Seq[CompanyChanceRecord] = {

    val tn = change_extract.tn
    val update_type = change_extract.update_type
    val old_data = change_extract.old_data
    var new_data = change_extract.new_data
    val biz_date = change_extract.biz_date
    val rowkey = change_extract.rowkey
    val update_time = change_extract.update_time
    val ids = companyIds.filter(_.length == 32).distinct
    val list = ListBuffer[CompanyChanceRecord]()
    if (ids.nonEmpty) {
      ids.foreach(id => {
        if (new_data.contains("case_amt")) {
          new_data += ("case_amt" -> BaseUtil.amt_div(BaseUtil.getOrEmptyStr(new_data, "case_amt"), "1").toString)
        }
        if (new_data.contains("judge_date")) {
          new_data += ("judge_date" -> BaseUtil.getOrEmptyStr(new_data, "judge_date").split(" ")(0))
        }
        import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._

        val record: CompanyChanceRecord = CompanyChanceRecord(
          id = CompanyChanceUtils.generateId(rowkey, biz_date, tn)
          , rowkey = rowkey
          , company_id = id
          , title = ""
          , label = get_label(new_data)
          , change_time = biz_date
          , update_time = update_time
          , update_type = update_type
          , deleted = get_deleted(new_data)
          , new_data.toJson
          , get_party_set(new_data).head
          , get_party_set(new_data).last
        )
        list.append(record)
      })
    }
    list
  }

}