package com.winhc.bigdata.spark.ng.chance

import cn.hutool.crypto.SecureUtil
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.Row

/**
 * @author: π
 * @date: 2021/9/28 13:49
 */
case class ChangeExtract(rowkey: String
                         , company_id: String
                         , company_name: String
                         , tn: String
                         , update_type: String
                         , old_data: Map[String, String]
                         , new_data: Map[String, String]
                         , change_fields: Seq[String]
                         , biz_date: String
                         , update_time: String
                        )


case class EntityInfo(keyno: String
                      , name: String
                     ) extends Comparable[EntityInfo] {


  override def hashCode(): Int = s"$name".hashCode()

  override def equals(obj: Any): Boolean =
    obj match {
      case d: EntityInfo =>
        s"$name".equals(s"${d.name}")
      case _ =>
        false
    }

  override def compareTo(o: EntityInfo): Int = s"$name".compareTo(s"${o.name}")
}

case class RowkeyInfo(rowkey: String, tn: String) {
  def toStr(): String = {
    s"$tn@@$rowkey"
  }
}


case class CompanyChanceRecord(id: String,
                               rowkey: String,
                               company_id: String,
                               title: String,
                               label: String,
                               change_time: String,
                               update_time: String,
                               update_type: String,
                               deleted: String,
                               new_data: String,
                               plaintiff: String,
                               defendant: String
                              ) {

  def format(): CompanyChanceRecord = {
    if (id == null) {
      return null
    }
    //剔除id为空的数据
    if (StringUtils.isBlank(company_id)) {
      return null
    }
    return CompanyChanceRecord(
      id,
      rowkey,
      company_id,
      title,
      label,
      change_time,
      update_time,
      update_type,
      deleted,
      new_data,
      plaintiff,
      defendant
    )
    this
  }

  def to_row(): Row = {
    Row(SecureUtil.md5(id)
      , rowkey
      , company_id
      , title
      , label
      , change_time
      , update_time
      , update_type
      , deleted
      , new_data
      , plaintiff
      , defendant
    )
  }

}


