package com.winhc.bigdata.spark.ng.chance.table

import com.winhc.bigdata.spark.implicits.MapHelper._
import com.winhc.bigdata.spark.ng.chance.{ChangeExtract, CompanyChanceHandle, CompanyChanceRecord, CompanyChanceUtils}
import org.json4s.DefaultFormats
import org.json4s.jackson.Json


/**
 * @Date: 2021/9/29
 * @Description: 破产公告
 */
case class bankruptcy_open_case(is_inc: Boolean) extends CompanyChanceHandle {

  override def get_conditional_filter(): String = {
    ""
  }

  override def get_label(new_data: Map[String, String]): String = {
    val m: Map[String, String] = Map(
      "agency_court" -> new_data.getOrEmptyStr("agency_court"),
      "case_type" -> new_data.getOrEmptyStr("case_type")
    )
    Json(DefaultFormats).write(m)
  }

  override def get_party_set(new_data: Map[String, String]): Seq[String] = {
    Seq(
      new_data.getOrEmptyStr("applicant"),
      new_data.getOrEmptyStr("respondent")
    )
  }

  override def filter: (String, String, Seq[String], Map[String, String], Map[String, String]) => Boolean = (update_type: String, biz_date: String, change_fields: Seq[String], old_data: Map[String, String], new_data: Map[String, String]) => {
    if (CompanyChanceUtils.default_filter(update_type, biz_date, change_fields, old_data, new_data)) {
      if (update_type.equals("insert")) true
      else
        false
    } else {
      false
    }
  }

  override def getCompanyIds(new_data: Map[String, String]): Seq[String] = {
    getEntity(new_data.getOrEmptyStr("respondent_info"), "litigant_id")
  }

  override def flat_map: ChangeExtract => Seq[CompanyChanceRecord] = (change_extract: ChangeExtract) => {
    getCompanyChanceRecord(change_extract, getCompanyIds(change_extract.new_data))
  }
}
