package com.winhc.bigdata.spark.ng.chance.table

import com.winhc.bigdata.spark.implicits.MapHelper._
import com.winhc.bigdata.spark.ng.chance.{ChangeExtract, CompanyChanceHandle, CompanyChanceRecord, CompanyChanceUtils}
import org.json4s.{DefaultFormats, Formats, NoTypeHints}
import org.json4s.jackson.{Json, Serialization}
import org.json4s.jackson.Serialization.write


/**
 * @Date: 2021/9/29
 * @Description: 法院公告-原告
 */
case class company_court_announcement(is_inc: Boolean) extends CompanyChanceHandle {

  override def get_conditional_filter(): String = {
    "AND  new_data['announcement_type'] = '起诉状副本及开庭传票' "
  }

  override def get_label(new_data: Map[String, String]): String = {
    val m: Map[String, String] = Map(
      "announcement_type" -> new_data.getOrEmptyStr("announcement_type"),
      "publish_date" -> new_data.getOrEmptyStr("publish_date")
    )
    Json(DefaultFormats).write(m)
  }

  override def get_party_set(new_data: Map[String, String]): Seq[String] = {
    Seq(
      new_data.getOrEmptyStr("plaintiff"),
      new_data.getOrEmptyStr("litigant")
    )
  }

  override def filter: (String, String, Seq[String], Map[String, String], Map[String, String]) => Boolean = (update_type: String, biz_date: String, change_fields: Seq[String], old_data: Map[String, String], new_data: Map[String, String]) => {
    if (CompanyChanceUtils.default_filter(update_type, biz_date, change_fields, old_data, new_data)) {
      if (update_type.equals("insert")) true
      else
        false
    } else {
      false
    }
  }

  override def getCompanyIds(new_data: Map[String, String]): Seq[String] = {
    //Seq(new_data.getOrEmptyStr("related_company_id")) ++
    getEntity(new_data.getOrEmptyStr("plaintiff_info"), "litigant_id") //++
    //getEntity(new_data.getOrEmptyStr("pledgee_info"), "pledgee_id")
  }

  override def flat_map: ChangeExtract => Seq[CompanyChanceRecord] = (change_extract: ChangeExtract) => {
    getCompanyChanceRecord(change_extract, getCompanyIds(change_extract.new_data))
  }
}
