package com.winhc.bigdata.spark.utils

import com.github.stuxuhai.jpinyin.ChineseHelper
import com.winhc.bigdata.spark.implicits.CaseClass2JsonHelper._
import com.winhc.bigdata.spark.utils.BaseUtil.cleanup
import org.apache.commons.lang3.StringUtils

/**
 * @author: XuJiakai
 * @date: 2020/11/23 10:46
 */

case class human(id: String, name: String) {
  def buildJson: String = {
    val simplifiedChinese = CompanyIndexUtils.convertToSimplifiedChinese(name)
    if (simplifiedChinese.equals(name)) {
      Map("id" -> id,
        "name" -> name
      ).toJson()
    } else {
      Map("id" -> id,
        "name" -> simplifiedChinese
      ).toJson()
    }
  }
}

case class holder(id: String, `type`: String, name: String) {
  def buildJson: String = {
    val simplifiedChinese = CompanyIndexUtils.convertToSimplifiedChinese(name)
    if (name.equals(simplifiedChinese)) {
      Map("id" -> id,
        "type" -> `type`,
        "name" -> name
      ).toJson()
    } else {
      Map("id" -> id,
        "type" -> `type`,
        "name" -> simplifiedChinese
      ).toJson()
    }
  }
}

case class CompanyName(show: String, value: String) {
  def buildJson: String = {
    val simplifiedChineseValue = CompanyIndexUtils.convertToSimplifiedChinese(value)
    val simplifiedChinese = CompanyIndexUtils.convertToSimplifiedChinese(show)

    if (show.equals(simplifiedChinese))
      Map("show" -> show,
        "value" -> value
      ).toJson()
    else
      Map("show" -> show,
        "value" -> simplifiedChineseValue,
        "simplified_chinese" -> simplifiedChinese
      ).toJson()
  }
}


object CompanyIndexUtils {
  def getHuman(id: String, name: String): String = human(id, name).buildJson

  def get_holder(id: String, `type`: String, name: String): String = holder(id, `type`, name).buildJson


  def getCompanyName(name: String): CompanyName = {
    if (StringUtils.isEmpty(name)) null
    else {
      val value = cleanup(name)
      CompanyName(name.replaceAll("\t;", "").trim, value)
    }
  }

  def getHistoryName(cname: String, names: String): Seq[CompanyName] = {
    if (StringUtils.isEmpty(names)) {
      null
    } else {
      val res = getSplit(names)
        .filter(StringUtils.isNotBlank(_))
        .map(StringUtils.trim(_))
        .filter(!cname.equals(_))
        .map(getCompanyName)
      if (res.isEmpty) {
        null
      } else {
        res
      }
    }
  }

  def company_score_weight(reg_status: String, cname: String, reg_capital_amount: String, company_type: String): String = {
    if (cname == null || cleanup(cname).replaceAll("[0-9]", "").length == 1)
      return "0.01"
    if (cleanup(cname).replaceAll("[0-9]", "").length <= 3)
      return "0.3"
    if (reg_status == null || reg_status.contains("销") || reg_status.contains("消"))
      return "1"
    val amount = reg_capital_amount == null match {
      case true => 0L
      case false => reg_capital_amount.toDouble
    }
    var w = Math.log(amount / 10000000 + 1) + 1
    if ("1".equals(company_type))
      w = w + 3
    w + ""
  }

  def getSplit(str: String): Seq[String] = {
    if (StringUtils.isNotBlank(str)) {
      str.split("\t;\t").filter(StringUtils.isNotBlank).toSet.toList
    } else {
      Seq.empty[String]
    }
  }

  def convertToSimplifiedChinese(name: String): String = {
    if (StringUtils.isEmpty(name)) null
    else ChineseHelper.convertToSimplifiedChinese(name)
  }


  def main(args: Array[String]): Unit = {
    //    println(company_score_weight("存续（在营、开业、在册）","新疆现代特油科技股份有限公司","200309577000000","1"))

    println(getCompanyName("香港中旅（中國）國際投資有限公司").buildJson)
  }

}
